class DAG:
    def __init__(self):
        self.tasks = {}

    def add_task(self, task):
        self.tasks[task.task_id] = task

    def set_downstream(self, task_id, next_task_id):
        task = self.tasks[task_id]
        next_task = self.tasks[next_task_id]
        task.next(next_task)

    def run(self, start_task_id):
        start_task = self.tasks[start_task_id]
        start_task.run()



from f_scheduler.operators.base_op import BaseOperator


class DefaultFunctionOperator(BaseOperator):
    def __init__(self, function, param, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.function = function
        self.param = param
        self.return_value = None

    def execute(self, context: dict):
        try:
            result = self.function(*self.param)
            if result is not None:
                self.return_value = result
                return result
            else:
                return True
        except Exception as e:
            print(f"Function execution failed with error: {e}")
        return False
